import { app } from '@/app.ts';
import { ChatServices } from './services/chat-services.ts';
import { ChatConfigServices } from './services/chat-config-srevices.ts';
import { AiChatHistoryModel } from './models/ai-chat-history.ts';
import { UserPermission } from '@kevisual/permission';
app
  .route({
    path: 'ai',
    key: 'chat',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const data = ctx.query.data || {};
    const { id, messages = [], options = {}, title, hook, getFull = false } = data;
    let { username, model, group } = ctx.query;
    const tokenUser = ctx.state.tokenUser || {};
    const tokenUsername = tokenUser.username;
    let aiChatHistory: AiChatHistoryModel;
    if (id) {
      aiChatHistory = await AiChatHistoryModel.findByPk(id);
      if (!aiChatHistory) {
        ctx.throw(400, 'aiChatHistory not found');
      }
      if (aiChatHistory.uid !== tokenUser.id) {
        ctx.throw(403, 'not permission');
      }
      username = username || aiChatHistory.username || tokenUsername;
      model = model || aiChatHistory.model;
      group = group || aiChatHistory.group;
    } else {
      username = username || tokenUsername;
    }
    const isSelf = username === tokenUsername;

    if (!Array.isArray(messages)) {
      ctx.throw(400, 'chat messages is not array');
    }
    // 初始化服务
    const chatServices = await ChatServices.createServices({
      owner: username,
      model,
      group,
      username: tokenUsername,
    });
    if (!isSelf && username !== 'root') {
      const aiConfig = chatServices.aiConfig;
      const permission = new UserPermission({ permission: aiConfig.permission, owner: username });
      const checkPermission = permission.checkPermissionSuccess({ username: tokenUsername, password: options.password });
      if (!checkPermission.success) {
        ctx.throw(403, checkPermission.message);
      }
    }

    const chatConfigServices = new ChatConfigServices(username, tokenUsername);
    await chatConfigServices.checkUserCanChat(tokenUsername);
    await chatServices.checkCanChat();
    const pickMessages = await chatServices.chatMessagePick(messages);
    if (pickMessages.length === 0) {
      ctx.throw(400, 'chat messages is empty');
    }
    const res = await chatServices.chat(pickMessages, options);
    if (!aiChatHistory) {
      aiChatHistory = await AiChatHistoryModel.create({
        username,
        model,
        group,
        title,
      });
      if (!title) {
        // TODO: 创建标题
      }
    }
    const message = res.choices[0].message;
    const newMessage = await chatServices.createNewMessage([...messages, message]);

    const usage = chatServices.chatProvider.getChatUsage();
    await chatServices.updateChatLimit(usage.total_tokens);
    await chatConfigServices.updateUserChatLimit(tokenUsername, usage.total_tokens);

    const needUpdateData: any = {
      messages: newMessage,
      prompt_tokens: aiChatHistory.prompt_tokens + usage.prompt_tokens,
      completion_tokens: aiChatHistory.completion_tokens + usage.completion_tokens,
      total_tokens: aiChatHistory.total_tokens + usage.total_tokens,
      version: aiChatHistory.version + 1,
      model: model,
      group: group,
      username: username,
    };
    if (hook) {
      needUpdateData.data = {
        ...aiChatHistory.data,
        hook,
      };
    }
    await AiChatHistoryModel.update(needUpdateData, { where: { id: aiChatHistory.id } });
    ctx.body = {
      message: newMessage[newMessage.length - 1],
      aiChatHistory: getFull || !id ? aiChatHistory : undefined,
    };
  })
  .addTo(app);

// http://localhost:4010/api/router?path=ai&key=question&question="1 and 1 equals"
app
  .route({
    path: 'ai',
    key: 'question',
    middleware: ['auth'],
    isDebug: true,
  })
  .define(async (ctx) => {
    const data = ctx.query;
    const model = data.model || 'qwq:latest';
    const group = data.group || 'ollama';
    const tokenUser = ctx.state.tokenUser;
    const chatServices = await ChatServices.createServices({
      owner: data.username || 'root',
      model,
      group,
      username: tokenUser.username,
    });
    const res = await chatServices.chat([
      {
        role: 'user',
        content: data.question,
      },
    ]);
    ctx.body = res;
  })
  .addTo(app);

// http://localhost:4010/api/router?path=ai&key=get-model-list
app
  .route({
    path: 'ai',
    key: 'get-model-list',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const username = ctx.query.username || 'root';
    const tokenUser = ctx.state.tokenUser;
    const usernames = ctx.query.data?.usernames || [];
    const tokenUsername = tokenUser.username;
    const isSameUser = username === tokenUser.username;
    const configArray: any[] = [];
    const services = new ChatConfigServices(username, tokenUser.username);
    const res = await services.getChatConfig(true, ctx.query.token);
    configArray.push({
      username,
      config: res,
    });
    if (!isSameUser) {
      const selfServices = new ChatConfigServices(tokenUser.username, tokenUser.username);
      const selfRes = await selfServices.getChatConfig(true, ctx.query.token);
      configArray.push({
        username: tokenUser.username,
        self: true,
        config: selfRes,
      });
    }
    for (const username of usernames) {
      const services = new ChatConfigServices(username, tokenUser.username);
      const res = await services.getChatConfig(true, ctx.query.token);
      const aiConfig = services.aiConfig;
      const permission = new UserPermission({ permission: aiConfig.permission, owner: username });
      const checkPermission = permission.checkPermissionSuccess({ username: tokenUsername, password: '-----------------' });
      if (!checkPermission.success) {
        // ctx.throw(403, `[${username}] ${checkPermission.message}`);
        configArray.push({
          username,
          config: null,
          error: checkPermission.message,
        });
      } else {
        configArray.push({
          username,
          config: res,
        });
      }
    }
    ctx.body = configArray;
  })
  .addTo(app);

app
  .route({
    path: 'ai',
    key: 'get-chat-usage',
    description: '获取chat使用情况， 只获取root的使用情况',
    middleware: ['auth'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser || {};
    const username = tokenUser.username;
    const services = new ChatConfigServices('root', username);
    const chatServices = await ChatServices.createServices({ owner: username, username });
    const rootUsage = await services.getUserChatLimit(username);
    const selfUsage = await chatServices.getChatLimit();

    ctx.body = {
      rootUsage,
      selfUsage,
    };
  })
  .addTo(app);
