import { AIConfig, AIConfigParser, ProviderResult } from '@/provider/utils/parse-config.ts';
import { ProviderManager, ChatMessage, BaseChat, ChatMessageOptions } from '@/provider/index.ts';
import { redis } from '@/modules/db.ts';
import { CustomError } from '@kevisual/router';
import { ChatConfigServices } from './chat-config-srevices.ts';
import { pick } from 'lodash-es';
import { ChastHistoryMessage } from '../models/ai-chat-history.ts';
import { nanoid } from '@/utils/uuid.ts';
import dayjs from 'dayjs';

export type ChatServicesConfig = {
  owner: string;
  model: string;
  group: string;
  decryptKey?: string;
};
export class ChatServices {
  cachePrefix = 'ai-chat:model:';
  /**
   * 用户名
   */
  owner: string;
  /**
   * 模型
   */
  model: string;
  /**
   * 组
   */
  group: string;
  /**
   * 解密密钥
   */
  decryptKey?: string;
  /**
   * 模型配置
   */
  modelConfig?: ProviderResult;
  aiConfig?: AIConfig;
  chatProvider?: BaseChat;
  constructor(opts: ChatServicesConfig) {
    this.owner = opts.owner;
    this.model = opts.model;
    this.group = opts.group;
    this.decryptKey = opts.decryptKey;
  }
  /**
   * 初始化
   * @returns
   */
  async init(username: string) {
    const config = await this.getConfig(username);
    const aiConfigParser = new AIConfigParser(config);
    const model = this.model;
    const group = this.group;
    const decryptKey = this.decryptKey;
    const providerResult = aiConfigParser.getProvider({ model, group, decryptKey });
    const that = this;
    const apiKey = await aiConfigParser.getSecretKey({
      getCache: async (key) => {
        const cache = await redis.get(that.wrapperKey(key));
        return cache ? JSON.parse(cache) : null;
      },
      setCache: async (key, value) => {
        await redis.set(that.wrapperKey(key), JSON.stringify(value), 'EX', 60 * 60 * 24 * 1); // 1天
      },
    });
    that.modelConfig = { ...providerResult, apiKey };
    that.aiConfig = config;
    return that.modelConfig;
  }
  /**
   * 包装key , 默认了username
   * @param key
   * @returns
   */
  wrapperKey(key: string) {
    const owner = this.owner;
    return `${this.cachePrefix}${owner}:${key}`;
  }
  async getConfig(username: string) {
    const services = new ChatConfigServices(this.owner, username);
    return services.getChatConfig();
  }

  async chat(messages: ChatMessage[], options?: ChatMessageOptions) {
    const { model, provider, apiKey, baseURL } = this.modelConfig;
    const providerManager = await ProviderManager.createProvider({
      provider: provider,
      model: model,
      apiKey: apiKey,
      baseURL: baseURL,
    });
    this.chatProvider = providerManager;
    const result = await providerManager.chat(messages, options);
    return result;
  }
  async createTitle(messages: ChastHistoryMessage[]) {
    return nanoid();
  }
  /**
   * 过滤消息，只保留对话需要的内容，name,role,content
   * @param messages
   * @returns
   */
  async chatMessagePick(messages: ChastHistoryMessage[]) {
    let newMessages = messages.filter((item) => !item.hide && !item.noUse);
    return newMessages.map((item) => pick(item, ['role', 'content', 'name'])) as ChatMessage[];
  }
  async createNewMessage(messages: ChastHistoryMessage[]) {
    return messages.map((item) => {
      if (!item.id) {
        item.id = 'chat-' + nanoid();
        item.createdAt = Date.now();
        item.updatedAt = Date.now();
      }
      return item;
    });
  }
  static async createServices(opts: Partial<ChatServicesConfig> & { username: string }) {
    const owner = opts.owner || 'root';
    const model = opts.model || 'deepseek-chat';
    const group = opts.group || 'deepseek';
    const decryptKey = opts.decryptKey;
    const chatServices = new ChatServices({ owner, model, group, decryptKey });
    await chatServices.init(opts.username);
    return chatServices;
  }

  /**
   * 检查模型的余量
   * @returns
   */
  async checkCanChat() {
    const { modelConfig } = this;
    const { tokenLimit, dayLimit, group, model } = modelConfig;
    const key = this.wrapperKey(`chat-limit`);
    const cache = await redis.get(key);
    if (cache) {
      const cacheData = JSON.parse(cache);
      const today = dayjs().format('YYYY-MM-DD');
      const current = cacheData.find((item) => item.group === group && item.model === model);
      const day = current[today] || 0;
      const token = current.token || 0;
      if (tokenLimit && token >= tokenLimit) {
        throw new CustomError(400, 'token limit exceeded');
      }
      if (dayLimit && day >= dayLimit) {
        throw new CustomError(400, 'day limit exceeded');
      }
    }
    return true;
  }
  /**
   * 获取模型的使用情况
   * @returns
   */
  async getChatLimit() {
    const { modelConfig } = this;
    const { group, model } = modelConfig;
    const key = this.wrapperKey(`chat-limit`);
    const cache = await redis.get(key);
    const today = dayjs().format('YYYY-MM-DD');
    if (cache) {
      const cacheData = JSON.parse(cache);
      return cacheData;
    }
    return [
      {
        group: group,
        model: model,
        token: 0,
        [today]: 0,
      },
    ];
  }
  /**
   * 更新模型的使用情况
   * @param token
   */
  async updateChatLimit(token: number) {
    const { modelConfig } = this;
    const { group, model } = modelConfig;
    const key = this.wrapperKey(`chat-limit`);
    const cache = await redis.get(key);
    const today = dayjs().format('YYYY-MM-DD');
    if (cache) {
      const cacheData = JSON.parse(cache);
      const current = cacheData.find((item) => item.group === group && item.model === model);
      if (current) {
        const day = current[today] || 0;
        current[today] = day + 1;
        current.token = current.token + token;
      } else {
        cacheData.push({ group, model, token: token, [today]: 1 });
      }
      await redis.set(key, JSON.stringify(cacheData), 'EX', 60 * 60 * 24 * 30); // 30天
    } else {
      await redis.set(key, JSON.stringify({ group, model, token: token, [today]: 1 }), 'EX', 60 * 60 * 24 * 30); // 30天
    }
  }
}
